"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const Request_1 = require("../src/Request");
const Response_1 = require("../src/Response");
const MiddlewareStore_1 = require("../src/MiddlewareStore");
const Server_1 = require("../src/Server");
const HttpContext_1 = require("../src/HttpContext");
class HttpServerProvider {
    constructor($container) {
        this.$container = $container;
    }
    $registerRequestResponse() {
        this.$container.bind('Adonis/Core/Request', () => Request_1.Request);
        this.$container.bind('Adonis/Core/Response', () => Response_1.Response);
    }
    $registerMiddlewareStore() {
        this.$container.bind('Adonis/Core/MiddlewareStore', () => MiddlewareStore_1.MiddlewareStore);
    }
    $registerHTTPContext() {
        this.$container.bind('Adonis/Core/HttpContext', () => HttpContext_1.HttpContext);
    }
    $registerHttpServer() {
        this.$container.singleton('Adonis/Core/Server', () => {
            const Logger = this.$container.use('Adonis/Core/Logger');
            const Profiler = this.$container.use('Adonis/Core/Profiler');
            const Config = this.$container.use('Adonis/Core/Config');
            const Encryption = this.$container.use('Adonis/Core/Encryption');
            const config = Object.assign({ secret: Config.get('app.appKey') }, Config.get('app.http', {}));
            return new Server_1.Server(this.$container, Logger, Profiler, Encryption, config);
        });
    }
    $registerRouter() {
        this.$container.singleton('Adonis/Core/Route', () => {
            const Server = this.$container.use('Adonis/Core/Server');
            return Server.router;
        });
    }
    register() {
        this.$registerRequestResponse();
        this.$registerMiddlewareStore();
        this.$registerHttpServer();
        this.$registerHTTPContext();
        this.$registerRouter();
    }
}
exports.default = HttpServerProvider;
