"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const qs_1 = __importDefault(require("qs"));
const cuid_1 = __importDefault(require("cuid"));
const fresh_1 = __importDefault(require("fresh"));
const net_1 = require("net");
const type_is_1 = __importDefault(require("type-is"));
const accepts_1 = __importDefault(require("accepts"));
const proxy_addr_1 = __importDefault(require("proxy-addr"));
const macroable_1 = require("macroable");
const lodash_1 = require("lodash");
const url_1 = require("url");
const cookie_1 = require("@poppinss/cookie");
class Request extends macroable_1.Macroable {
    constructor(request, response, _encryption, _config) {
        super();
        this.request = request;
        this.response = response;
        this._encryption = _encryption;
        this._config = _config;
        this.parsedUrl = url_1.parse(this.request.url, false);
        this._body = {};
        this._all = {};
        this._original = {};
        this._qs = {};
        this._raw = null;
        this._lazyAccepts = null;
        this._parsedCookies = null;
        this._parseQueryString();
    }
    _parseQueryString() {
        if (this.parsedUrl.query) {
            this.updateQs(qs_1.default.parse(this.parsedUrl.query));
            this._original = { ...this._all };
        }
    }
    _parseCookies() {
        if (!this._parsedCookies) {
            this._parsedCookies = cookie_1.parse(this.header('cookie'), this._config.secret);
        }
    }
    _initiateAccepts() {
        this._lazyAccepts = this._lazyAccepts || accepts_1.default(this.request);
    }
    id() {
        let requestId = this.header('x-request-id');
        if (!requestId && this._config.generateRequestId) {
            requestId = cuid_1.default();
            this.request.headers['x-request-id'] = requestId;
        }
        return requestId;
    }
    setInitialBody(body) {
        if (this._original && Object.isFrozen(this._original)) {
            throw new Error('Cannot re-set initial body. Use request.updateBody instead');
        }
        this.updateBody(body);
        this._original = Object.freeze({ ...this._all });
    }
    updateBody(body) {
        this._body = body;
        this._all = { ...this._body, ...this._qs };
    }
    updateRawBody(rawBody) {
        this._raw = rawBody;
    }
    updateQs(data) {
        this._qs = data;
        this._all = { ...this._body, ...this._qs };
    }
    get() {
        return this._qs;
    }
    post() {
        return this._body;
    }
    all() {
        return this._all;
    }
    original() {
        return this._original;
    }
    raw() {
        return this._raw;
    }
    input(key, defaultValue) {
        return lodash_1.get(this._all, key, defaultValue);
    }
    except(keys) {
        return lodash_1.omit(this._all, keys);
    }
    only(keys) {
        return lodash_1.pick(this._all, keys);
    }
    intended() {
        return this.request.method;
    }
    method() {
        if (this._config.allowMethodSpoofing && this.intended() === 'POST') {
            return this.input('_method', this.intended()).toUpperCase();
        }
        return this.intended();
    }
    headers() {
        return this.request.headers;
    }
    header(key, defaultValue) {
        key = key.toLowerCase();
        const headers = this.headers();
        switch (key) {
            case 'referer':
            case 'referrer':
                return headers.referrer || headers.referer || defaultValue;
            default:
                return headers[key] || defaultValue;
        }
    }
    ip() {
        const ipFn = this._config.getIp;
        if (typeof (ipFn) === 'function') {
            return ipFn(this);
        }
        return proxy_addr_1.default(this.request, this._config.trustProxy);
    }
    ips() {
        return proxy_addr_1.default.all(this.request, this._config.trustProxy);
    }
    protocol() {
        if (this.request.connection['encrypted']) {
            return 'https';
        }
        if (!this._config.trustProxy(this.request.connection.remoteAddress, 0)) {
            return this.parsedUrl.protocol || 'http';
        }
        const forwardedProtocol = this.header('X-Forwarded-Proto');
        return forwardedProtocol ? forwardedProtocol.split(/\s*,\s*/)[0] : 'http';
    }
    secure() {
        return this.protocol() === 'https';
    }
    hostname() {
        let host = this.header('host');
        if (this._config.trustProxy(this.request.connection.remoteAddress, 0)) {
            host = this.header('X-Forwarded-Host') || host;
        }
        if (!host) {
            return null;
        }
        const offset = host[0] === '[' ? host.indexOf(']') + 1 : 0;
        const index = host.indexOf(':', offset);
        return index !== -1 ? host.substring(0, index) : host;
    }
    subdomains() {
        const hostname = this.hostname();
        if (!hostname || net_1.isIP(hostname)) {
            return [];
        }
        const offset = this._config.subdomainOffset;
        const subdomains = hostname.split('.').reverse().slice(offset);
        if (subdomains[subdomains.length - 1] === 'www') {
            subdomains.splice(subdomains.length - 1, 1);
        }
        return subdomains;
    }
    ajax() {
        const xRequestedWith = this.header('X-Requested-With', '');
        return xRequestedWith.toLowerCase() === 'xmlhttprequest';
    }
    pjax() {
        return !!this.header('X-Pjax');
    }
    url(includeQueryString) {
        const pathname = this.parsedUrl.pathname;
        return includeQueryString && this.parsedUrl.query
            ? `${pathname}?${this.parsedUrl.query}`
            : pathname;
    }
    completeUrl(includeQueryString) {
        const protocol = this.protocol();
        const hostname = this.hostname();
        return `${protocol}://${hostname}${this.url(includeQueryString)}`;
    }
    is(types) {
        return type_is_1.default(this.request, types) || null;
    }
    accepts(types) {
        this._initiateAccepts();
        return this._lazyAccepts.type(types) || null;
    }
    types() {
        this._initiateAccepts();
        return this._lazyAccepts.types();
    }
    language(languages) {
        this._initiateAccepts();
        return this._lazyAccepts.language(languages) || null;
    }
    languages() {
        this._initiateAccepts();
        return this._lazyAccepts.languages();
    }
    charset(charsets) {
        this._initiateAccepts();
        return this._lazyAccepts.charset(charsets) || null;
    }
    charsets() {
        this._initiateAccepts();
        return this._lazyAccepts.charsets();
    }
    encoding(encodings) {
        this._initiateAccepts();
        return this._lazyAccepts.encoding(encodings) || null;
    }
    encodings() {
        this._initiateAccepts();
        return this._lazyAccepts.encodings();
    }
    hasBody() {
        return type_is_1.default.hasBody(this.request);
    }
    fresh() {
        if (['GET', 'HEAD'].indexOf(this.intended()) === -1) {
            return false;
        }
        const status = this.response.statusCode;
        if ((status >= 200 && status < 300) || status === 304) {
            return fresh_1.default(this.headers(), this.response.getHeaders());
        }
        return false;
    }
    stale() {
        return !this.fresh();
    }
    cookies() {
        this._parseCookies();
        return this._parsedCookies.signedCookies;
    }
    cookie(key, defaultValue) {
        this._parseCookies();
        return lodash_1.get(this._parsedCookies.signedCookies, key, defaultValue);
    }
    plainCookies() {
        this._parseCookies();
        return this._parsedCookies.plainCookies;
    }
    plainCookie(key, defaultValue) {
        this._parseCookies();
        return lodash_1.get(this._parsedCookies.plainCookies, key, defaultValue);
    }
    hasValidSignature() {
        const { signature, ...rest } = this.get();
        if (!signature) {
            return false;
        }
        const signedUrl = this._encryption.child({ hmac: false }).decrypt(signature);
        if (!signedUrl) {
            return false;
        }
        if (rest.expires_at && Number(rest.expires_at) < Date.now()) {
            return false;
        }
        const queryString = qs_1.default.stringify(rest);
        return queryString ? `${this.url()}?${queryString}` === signedUrl : this.url() === signedUrl;
    }
    toJSON() {
        return {
            id: this.id(),
            url: this.url(),
            query: this.parsedUrl.query,
            headers: this.headers(),
            method: this.method(),
            protocol: this.protocol(),
        };
    }
}
exports.Request = Request;
Request._macros = {};
Request._getters = {};
