"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const utils_1 = require("@poppinss/utils");
const Group_1 = require("./Router/Group");
const BriskRoute_1 = require("./Router/BriskRoute");
const Resource_1 = require("./Router/Resource");
function dropSlash(input) {
    if (input === '/') {
        return '/';
    }
    return `/${input.replace(/^\//, '').replace(/\/$/, '')}`;
}
exports.dropSlash = dropSlash;
function toRoutesJSON(routes) {
    return routes.reduce((list, route) => {
        if (route instanceof Group_1.RouteGroup) {
            list = list.concat(toRoutesJSON(route.routes));
            return list;
        }
        if (route instanceof Resource_1.RouteResource) {
            list = list.concat(toRoutesJSON(route.routes));
            return list;
        }
        if (route instanceof BriskRoute_1.BriskRoute) {
            if (route.route) {
                list.push(route.route.toJSON());
            }
            return list;
        }
        if (!route.deleted) {
            list.push(route.toJSON());
        }
        return list;
    }, []);
}
exports.toRoutesJSON = toRoutesJSON;
function processPattern(pattern, data) {
    let url = pattern;
    if (url.indexOf(':') > -1) {
        const tokens = url.split('/');
        url = tokens.map((token) => {
            if (!token.startsWith(':')) {
                return token;
            }
            const isOptional = token.endsWith('?');
            const paramName = token.replace(/^:/, '').replace(/\?$/, '');
            const param = data[paramName];
            if (!param && !isOptional) {
                throw new utils_1.Exception(`\`${paramName}\` param is required to make URL for \`${pattern}\` route`, 500, 'E_MISSING_ROUTE_PARAM_VALUE');
            }
            return param;
        }).join('/');
    }
    return url;
}
exports.processPattern = processPattern;
function useReturnValue(returnValue, ctx) {
    return (returnValue !== undefined &&
        returnValue !== ctx.response &&
        !ctx.response.hasLazyBody);
}
exports.useReturnValue = useReturnValue;
